@@ -1,18 +1,18 @@
 package Alvis::TermTagger;
 
-our $VERSION = '0.8';
+our $VERSION = '0.81';
 
 #######################################################################
 #
-# Last Update: 21/12/2010 (mm/dd/yyyy date format)
+# Last Update: 16/09/2015 (mm/dd/yyyy date format)
 # 
 # Copyright (C) 2006 Thierry Hamon
 #
-# Written by thierry.hamon@univ-paris13.fr
+# Written by thierry.hamon@limsi.fr
 #
 # Author : Thierry Hamon
-# Email : thierry.hamon@univ-paris13.fr
-# URL : http://www-limbio.smbh.univ-paris13.fr/membres/hamon/
+# Email : thierry.hamon@limsi.fr
+# URL : https://perso.limsi.fr/hamon/
 #
 ########################################################################
 
@@ -20,6 +20,8 @@ our $VERSION = '0.8';
 use strict;
 use warnings;
 
+use utf8;
+
 # TODO : write functions for term tagginga, term selection with and
 # without offset in the corpus
 
@@ -66,6 +68,49 @@ sub termtagging {
     return(0);
 }
 
+sub termtagging_brat {
+
+    my ($corpus_filename, $term_list_filename, $output_filename, $lemmatised_corpus_filename, $caseSensitive) = @_;
+
+    my @term_list;
+    my %term_listIdx;
+    my @regex_term_list;
+    my @regex_lemmawordterm_list;
+    my %corpus;
+    my %lc_corpus;
+    my %lemmatised_corpus;
+    my %lc_lemmatised_corpus;
+    my %corpus_index;
+    my %lemmatised_corpus_index;
+    my %idtrm_select;
+    my %idlemtrm_select;
+
+    if (!defined $caseSensitive) {
+	$caseSensitive = -1;
+    }
+
+    &load_TermList($term_list_filename,\@term_list, \%term_listIdx);
+    &get_Regex_TermList(\@term_list, \@regex_term_list, \@regex_lemmawordterm_list);
+
+    &load_Corpus($corpus_filename, \%corpus, \%lc_corpus);
+    if (defined $lemmatised_corpus_filename) {
+	&load_Corpus($lemmatised_corpus_filename, \%lemmatised_corpus, \%lc_lemmatised_corpus);
+    }
+    &corpus_Indexing(\%lc_corpus, \%corpus, \%corpus_index, $caseSensitive);
+    if (defined $lemmatised_corpus_filename) {
+	&corpus_Indexing(\%lc_lemmatised_corpus, \%lemmatised_corpus, \%lemmatised_corpus_index, $caseSensitive);
+    }
+    &term_Selection(\%corpus_index, \@term_list, \%idtrm_select, $caseSensitive);
+    if (defined $lemmatised_corpus_filename) {
+	&term_Selection(\%lemmatised_corpus_index, \@term_list, \%idlemtrm_select, $caseSensitive);
+    }
+    &term_tagging_offset_brat(\@term_list, \@regex_term_list, \%idtrm_select, \%corpus, $output_filename, $caseSensitive);
+    if (defined $lemmatised_corpus_filename) {
+	&term_tagging_offset_brat(\@term_list, \@regex_lemmawordterm_list, \%idlemtrm_select, \%lemmatised_corpus, $output_filename, $caseSensitive);
+    }
+    return(0);
+}
+
 
 sub load_TermList {
     my ($termlist_name, $ref_termlist, $ref_termlistIdx) = @_;
@@ -97,6 +142,7 @@ sub load_TermList {
 		 $tab[0] =~ s/ $//go;
 		 $tab[0] =~ s/^ //go;
 #		 $tab[0] =~ s/\\:/:/go;
+		 # warn "term: " . $tab[0] . "\n";;
 		 if (!exists $ref_termlistIdx->{$tab[0]}) {
 		     push @$ref_termlist, \@tab;
 		     $ref_termlistIdx->{$tab[0]} = scalar(@$ref_termlist) -1;
@@ -122,6 +168,7 @@ sub get_Regex_TermList {
 	if (defined $ref_regex_lemmaWordtermlist) {
 	    if (defined $ref_termlist->[$term_counter]->[3]) {
 		$ref_regex_lemmaWordtermlist->[$term_counter] = $ref_termlist->[$term_counter]->[3];
+		# warn "==> " . $ref_termlist->[$term_counter]->[3] . "\n";
 	    } else {
 		$ref_regex_lemmaWordtermlist->[$term_counter] = $ref_termlist->[$term_counter]->[0];
 	    }
@@ -143,9 +190,11 @@ sub get_Regex_TermList {
 
 sub load_Corpus {
 
-    my ($corpus_filename,$ref_tabh_Corpus, $ref_tabh_Corpus_lc) = @_;
+    my ($corpus_filename, $ref_tabh_Corpus, $ref_tabh_Corpus_lc) = @_;
     my $line;
     my $sent_id = 1;
+    my $offset = 0;
+    my $lineLen = 0;
 
     warn "Loading the corpus\n";
 
@@ -154,10 +203,15 @@ sub load_Corpus {
     binmode(CORPUS, ":utf8");
     
     while($line=<CORPUS>){
+	$lineLen = length($line);
 	chomp $line;
-	$ref_tabh_Corpus->{$sent_id} = $line;
-	$ref_tabh_Corpus_lc->{$sent_id} = lc $line;	
+	$ref_tabh_Corpus->{$sent_id}->{'line'} = $line;
+	$ref_tabh_Corpus->{$sent_id}->{'offset'} = $offset;
+	$ref_tabh_Corpus_lc->{$sent_id}->{'line'} = lc $line;	
+	$ref_tabh_Corpus_lc->{$sent_id}->{'offset'} = $offset;	
+	# warn "=> " . $ref_tabh_Corpus_lc->{$sent_id} . "\n";
 	$sent_id++;
+	$offset += $lineLen;
     }
     close CORPUS;
     print STDERR "\n\tCorpus size : " . scalar(keys %$ref_tabh_Corpus) . "\n\n";
@@ -176,8 +230,8 @@ sub corpus_Indexing {
     warn "Indexing the corpus\n";
 
     foreach $sent_id (keys %$ref_corpus_lc) { # \-\.,\n;\/
-	@tab_words = split /[ ()\',\[\]\?\!:;\/\.\+\-\*\#\{\}\n]/, $ref_corpus->{$sent_id};
-	@tab_words_lc = split /[ ()\',\[\]\?\!:;\/\.\+\-\*\#\{\}\n]/, $ref_corpus_lc->{$sent_id};
+	@tab_words = split /[ ()\',\[\]\?\!:;\/\.\+\-\*\#\{\}\n]/, $ref_corpus->{$sent_id}->{'line'};
+	@tab_words_lc = split /[ ()\',\[\]\?\!:;\/\.\+\-\*\#\{\}\n]/, $ref_corpus_lc->{$sent_id}->{'line'};
 	for($i=0;$i < scalar(@tab_words_lc);$i++) {
 #	foreach $word_lc (@tab_words_lc) {
 	    if ((defined $caseSensitive) && (($caseSensitive == 0) || (length($tab_words_lc[$i]) <= $caseSensitive))) {
@@ -194,6 +248,8 @@ sub corpus_Indexing {
 	    }
 	}
     }
+    # print STDERR join(" : ", keys(%$ref_corpus_index)) . "\n";
+
     print STDERR "\n\tSize of the first selected term list: " . scalar(keys %$ref_corpus_index) . "\n\n";
 }
 
@@ -250,7 +306,7 @@ sub _term_Selection2 {
 }
 
 sub term_Selection {
-    my ($ref_corpus_index, $ref_termlist, $ref_tabh_idtrm_select, $caseSensitive) = @_;
+    my ($ref_corpus_index, $ref_termlist, $ref_tabh_idtrm_select, $caseSensitive, $termField) = @_;
     my $counter;
     my $term;
     my @tab_termlex;
@@ -263,60 +319,78 @@ sub term_Selection {
 
     my @recordedWords;
 
-    warn "Selecting the terms potentialy appearing in the corpus\n";
+    if (!defined $termField) {
+	$termField = 0;
+    }
+
+    warn "Selecting the terms potentialy appearing in the corpus ($termField)\n";
 
     my %tabh_numtrm_select;
     
     # warn "caseSensitive: $caseSensitive\n";
     for($counter  = 0;$counter < scalar @$ref_termlist;$counter++) {
-	if ((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[0]) <= $caseSensitive))) {
-	    $term = $ref_termlist->[$counter]->[0];
-	    $termCap = $ref_termlist->[$counter]->[0];
-	    # warn "passe\n";
+	if (defined $ref_termlist->[$counter]->[$termField]) {
+	    # warn "==> " . $ref_termlist->[$counter]->[0] . " / " . $ref_termlist->[$counter]->[$termField] . "\n";
+	    if ((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField]) <= $caseSensitive))) {
+		$term = $ref_termlist->[$counter]->[$termField];
+		$termCap = $ref_termlist->[$counter]->[$termField];
+		# warn "passe\n";
+	    } else {
+		$term = lc $ref_termlist->[$counter]->[$termField];
+		$termCap = $ref_termlist->[$counter]->[$termField];
+	    }
 	} else {
-	    $term = lc $ref_termlist->[$counter]->[0];
-	    $termCap = $ref_termlist->[$counter]->[0];
+		$term = lc $ref_termlist->[$counter]->[0];
+		$termCap = $ref_termlist->[$counter]->[0];
 	}
-	# warn "+++> $term ($termCap)\n";
-        # XXX - ABREVIATION - XXX
-	# @tab_termlex = split /[ \-:]+/, $term;
-	@tab_termlex = split /[ ()\',\[\]\?\!:;\/\.\+\-\*\#\{\}\n]+/, $term;
-	@tab_termlexCap = split /[ ()\',\[\]\?\!:;\/\.\+\-\*\#\{\}\n]+/, $termCap;
-	# @tab_termlex = split /[ \-:]+/, $term;
-	# @tab_termlexCap = split /[ \-:]+/, $termCap;
-	$word_found = 0;
-	$i=0; 
-	@recordedWords = ();
-	$word = $tab_termlex[$i];
-	# warn join(':', @tab_termlex) . " -- " . join(':', @tab_termlexCap) . "\n";
-	# warn scalar(@tab_termlex) . " -- " . scalar(@tab_termlexCap) . " ($i)\n";
-	while(($i < scalar(@tab_termlex)) && ($i < scalar(@tab_termlexCap)) && 
-	      ((($word eq "") || (exists $ref_corpus_index->{$word})) ||
-	      ((($caseSensitive == 0) || (length($ref_termlist->[$counter]->[0]) > $caseSensitive)) &&
-	       (exists $ref_corpus_index->{$tab_termlexCap[$i]})))
-	    ) {
-	    if ($word ne "") {
-		# warn "---> $term\n";
-		push @recordedWords, $word;
-	    # } else {
-	    # 	warn "--------------------------> $term\n";
-	    }
-	    $i++;
+	    # warn "+++> $term ($termCap)\n";
+	    # XXX - ABREVIATION - XXX
+	    # @tab_termlex = split /[ \-:]+/, $term;
+	    @tab_termlex = split /[ ()\',\[\]\?\!:;\/\.\+\-\*\#\{\}\n]+/, $term;
+	    @tab_termlexCap = split /[ ()\',\[\]\?\!:;\/\.\+\-\*\#\{\}\n]+/, $termCap;
+	    # @tab_termlex = split /[ \-:]+/, $term;
+	    # @tab_termlexCap = split /[ \-:]+/, $termCap;
+	    $word_found = 0;
+	    $i=0; 
+	    @recordedWords = ();
 	    $word = $tab_termlex[$i];
-	    # warn "i: $i\n";
-	}
-	if ($i == scalar(@tab_termlex)) {
-	    foreach $word (@recordedWords) {
-		if (!exists $ref_tabh_idtrm_select->{$counter}) {
-		    my %tabhtmp2;
-		    $ref_tabh_idtrm_select->{$counter} = \%tabhtmp2;
+	    # warn join(':', @tab_termlex) . " -- " . join(':', @tab_termlexCap) . "\n";
+	    # warn scalar(@tab_termlex) . " -- " . scalar(@tab_termlexCap) . " ($i)\n";
+	    while(($i < scalar(@tab_termlex)) && ($i < scalar(@tab_termlexCap)) && 
+		  ((($word eq "") || (exists $ref_corpus_index->{$word})) ||
+		   ((($caseSensitive == 0) || (length($termCap) > $caseSensitive)) &&
+		    (exists $ref_corpus_index->{$tab_termlexCap[$i]})))
+		) {
+#		   ((($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField]) > $caseSensitive)) &&
+
+		if ($word ne "") {
+		    # warn "---> $term\n";
+		    push @recordedWords, $word;
+		    # } else {
+		    # 	warn "--------------------------> $term\n";
 		}
-		foreach $sent_id (@{$ref_corpus_index->{$word}}) {
-		    ${$ref_tabh_idtrm_select->{$counter}}{$sent_id} = 1;
+		$i++;
+		$word = $tab_termlex[$i];
+		# warn "i: $i\n";
+	    }
+	    if ($i == scalar(@tab_termlex)) {
+		foreach $word (@recordedWords) {
+		    # print STDERR "$word : ";
+		    if (!exists $ref_tabh_idtrm_select->{$counter}) {
+			my %tabhtmp2;
+			$ref_tabh_idtrm_select->{$counter} = \%tabhtmp2;
+		    }
+		    foreach $sent_id (@{$ref_corpus_index->{$word}}) {
+			${$ref_tabh_idtrm_select->{$counter}}{$sent_id} = 1;
+		    }
 		}
 	    }
-	}
+#	}
     }
+    # print STDERR "\n";
+
+    # print STDERR join(" : ", keys(%$ref_tabh_idtrm_select)) . "\n";
+
     warn "Size of the selected list: " . scalar (keys %$ref_tabh_idtrm_select) . "\n";
     # foreach $counter (keys %$ref_tabh_idtrm_select) {
     # 	warn $ref_termlist->[$counter]->[0] . "\n";
@@ -326,12 +400,16 @@ sub term_Selection {
 }
 
 sub term_tagging_offset {
-    my ($ref_termlist, $ref_regex_termlist, $ref_tabh_idtrm_select, $ref_tabh_corpus, $offset_tagged_corpus_name, $caseSensitive) = @_;
+    my ($ref_termlist, $ref_regex_termlist, $ref_tabh_idtrm_select, $ref_tabh_corpus, $offset_tagged_corpus_name, $caseSensitive, $termField) = @_;
     my $counter;
     my $term_regex;
     my $sent_id;
     my $line;
+    my $termField2;
 
+    if (!defined $termField) {
+	$termField = 0;
+    }
     # XXX - ABREVIATION - XXX => regex
 
     warn "Term tagging\n";
@@ -342,28 +420,38 @@ sub term_tagging_offset {
 
     foreach $counter (keys %$ref_tabh_idtrm_select) {
 	$term_regex = $ref_regex_termlist->[$counter];
+	$termField2 = 0;
+	if (defined $ref_termlist->[$counter]->[$termField]) {
+	    $termField2 = $termField;
+	}
 	foreach $sent_id (keys %{$ref_tabh_idtrm_select->{$counter}}){
-	    $line = $ref_tabh_corpus->{$sent_id};
+	    $line = $ref_tabh_corpus->{$sent_id}->{'line'};
 	    print STDERR ".";
 	    
-	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[0]) <= $caseSensitive))) &&
+	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
 		 ($line =~ /[,.?!:;\/ \n\-\/\*'\#\{\}\(\)\[\]\+]($term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/)) || 
-		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[0]) > $caseSensitive)) && 
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
 		 ($line =~ /[,.?!:;\/ \n\-\/\*'\#\{\}\(\)\[\]\+]($term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/i))) {
 		printMatchingTerm(\*TAGGEDCORPUS, $ref_termlist->[$counter], $sent_id);
 	    }
-	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[0]) <= $caseSensitive))) &&
+	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
 		 ($line =~ /^($term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/i)) || 
-		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[0]) > $caseSensitive)) && 
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
 		 ($line =~ /^($term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/i))) {
 		printMatchingTerm(\*TAGGEDCORPUS, $ref_termlist->[$counter], $sent_id);
 	    }
-	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[0]) <= $caseSensitive))) &&
+	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
 		 ($line =~ /[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]($term_regex)$/)) ||
-		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[0]) > $caseSensitive)) && 
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
 		 ($line =~ /[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]($term_regex)$/i))) {
 		printMatchingTerm(\*TAGGEDCORPUS, $ref_termlist->[$counter], $sent_id);
 	    }
+	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
+		 ($line =~ /^($term_regex)$/i)) || 
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
+		 ($line =~ /^($term_regex)$/i))) {
+		printMatchingTerm(\*TAGGEDCORPUS, $ref_termlist->[$counter], $sent_id);
+	    }
 	}
 	print STDERR "\n";
     }
@@ -385,47 +473,63 @@ sub printMatchingTerm() {
 
 
 sub term_tagging_offset_tab {
-    my ($ref_termlist, $ref_regex_termlist, $ref_tabh_idtrm_select, $ref_tabh_corpus, $ref_tab_results, $caseSensitive) = @_;
+    my ($ref_termlist, $ref_regex_termlist, $ref_tabh_idtrm_select, $ref_tabh_corpus, $ref_tab_results, $caseSensitive, $termField) = @_;
     my $counter;
     my $term_regex;
     my $sent_id;
     my $line;
     my $i;
     my $size_termselect = scalar(keys %$ref_tabh_idtrm_select);
+    my $termField2;
 
     $i = 0;
 
+    if (!defined $termField) {
+	$termField = 0;
+    }
+
     # XXX - ABREVIATION - XXX => regex
-#    warn "====> $caseSensitive\n";
+    # warn "====> $caseSensitive\n";
     
     foreach $counter (keys %$ref_tabh_idtrm_select) {
-  	printf STDERR "Term tagging... %0.1f%%\r", ($i/$size_termselect)*100 ;
+#  	printf STDERR "Term tagging... %0.1f%%\r", ($i/$size_termselect)*100 ;
 	$term_regex = $ref_regex_termlist->[$counter];
 	# warn "counter: $counter ($term_regex)\n";
 
+	$termField2 = 0;
+	if (defined $ref_termlist->[$counter]->[$termField]) {
+	    $termField2 = $termField;
+	}
+
 	foreach $sent_id (keys %{$ref_tabh_idtrm_select->{$counter}}){
-	    $line = $ref_tabh_corpus->{$sent_id};
+	    $line = $ref_tabh_corpus->{$sent_id}->{'line'};
 
 	    # warn "$line\n$term_regex\n";
 
-	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[0]) <= $caseSensitive))) &&
+	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
 		 ($line =~ /[,.?!:;\/ \n\-\/\*'\#\{\}\(\)\[\]\+](?<term>$term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/s)) ||
-		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[0]) > $caseSensitive)) && 
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
 		 ($line =~ /[,.?!:;\/ \n\-\/\*'\#\{\}\(\)\[\]\+](?<term>$term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/is))) {
  		printMatchingTerm_tab($ref_termlist->[$counter], $+{term},  $sent_id, $ref_tab_results);
 	    }
- 	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[0]) <= $caseSensitive))) &&
+ 	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
 		 ($line =~ /^(?<term>$term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/s)) ||
-		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[0]) > $caseSensitive)) && 
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
 		 ($line =~ /^(?<term>$term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/is))) {
 		printMatchingTerm_tab($ref_termlist->[$counter], $+{term}, $sent_id, $ref_tab_results);
 	    }
-	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[0]) <= $caseSensitive))) &&
+	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
 		 ($line =~ /[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+](?<term>$term_regex)$/s)) ||
-		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[0]) > $caseSensitive)) && 
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
 		 ($line =~ /[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+](?<term>$term_regex)$/is))) {
 		printMatchingTerm_tab($ref_termlist->[$counter], $+{term}, $sent_id, $ref_tab_results);
 	    }
+ 	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
+		 ($line =~ /^(?<term>$term_regex)$/s)) ||
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
+		 ($line =~ /^(?<term>$term_regex)$/is))) {
+		printMatchingTerm_tab($ref_termlist->[$counter], $+{term}, $sent_id, $ref_tab_results);
+	    }
 	}
 	$i++;
     }
@@ -435,18 +539,112 @@ sub term_tagging_offset_tab {
     warn "\nEnd of term tagging\n";
 }
 
+sub term_tagging_offset_brat {
+    my ($ref_termlist, $ref_regex_termlist, $ref_tabh_idtrm_select, $ref_tabh_corpus, $offset_tagged_corpus_name, $caseSensitive, $termField) = @_;
+    my $counter;
+    my $term_regex;
+    my $sent_id;
+    my $line;
+    my $i;
+    my $size_termselect = scalar(keys %$ref_tabh_idtrm_select);
+    my $termField2;
+    my $termId = 1;
+    my $offset;
+    my $currOffset;
+
+    $i = 0;
+
+    warn "Term tagging ($offset_tagged_corpus_name)\n";
+
+    open TAGGEDCORPUS, ">$offset_tagged_corpus_name" or die "$0: $offset_tagged_corpus_name: No such file\n";
+
+    binmode(TAGGEDCORPUS, ":utf8");
+
+
+    if (!defined $termField) {
+	$termField = 0;
+    }
+
+    # XXX - ABREVIATION - XXX => regex
+    # warn "====> $caseSensitive\n";
+    
+    foreach $counter (keys %$ref_tabh_idtrm_select) {
+#  	printf STDERR "Term tagging... %0.1f%%\r", ($i/$size_termselect)*100 ;
+	$term_regex = $ref_regex_termlist->[$counter];
+	# warn "counter: $counter ($term_regex)\n";
+
+	$termField2 = 0;
+	if (defined $ref_termlist->[$counter]->[$termField]) {
+	    $termField2 = $termField;
+	}
+
+	foreach $sent_id (keys %{$ref_tabh_idtrm_select->{$counter}}){
+	    $line = $ref_tabh_corpus->{$sent_id}->{'line'};
+	    $offset = $ref_tabh_corpus->{$sent_id}->{'offset'};
+
+	    # warn "$line\n$term_regex\n";
+	    # warn "$line\n$offset\n";
+
+	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
+		 ($line =~ /(?<before>[,.?!:;\/ \n\-\/\*'\#\{\}\(\)\[\]\+])(?<term>$term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/s)) ||
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
+		 ($line =~ /(?<before>[,.?!:;\/ \n\-\/\*'\#\{\}\(\)\[\]\+])(?<term>$term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/is))) {
+		$currOffset = $offset+length($`)+length($+{before});
+ 		print_brat_output(\*TAGGEDCORPUS, \$termId, $+{term}, $currOffset, $currOffset + length($+{term}));
+	    }
+ 	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
+		 ($line =~ /^(?<term>$term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/s)) ||
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
+		 ($line =~ /^(?<term>$term_regex)[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+]/is))) {
+		$currOffset = $offset+length($`);
+		print_brat_output(\*TAGGEDCORPUS, \$termId, $+{term}, $currOffset, $currOffset + length($+{term}));
+	    }
+	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
+		 ($line =~ /(?<before>[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+])(?<term>$term_regex)$/s)) ||
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
+		 ($line =~ /(?<before>[,.?!:;\/ \n\-\/\*'\#\(\)\[\]\{\}\+])(?<term>$term_regex)$/is))) {
+		$currOffset = $offset+length($`)+length($+{before});
+		print_brat_output(\*TAGGEDCORPUS, \$termId, $+{term}, $currOffset, $currOffset+length($+{term}));
+	    }
+ 	    if ((((defined $caseSensitive) && (($caseSensitive == 0) || (length($ref_termlist->[$counter]->[$termField2]) <= $caseSensitive))) &&
+		 ($line =~ /^(?<term>$term_regex)$/s)) ||
+		(((!defined $caseSensitive) || ($caseSensitive < 0) || (length($ref_termlist->[$counter]->[$termField2]) > $caseSensitive)) && 
+		 ($line =~ /^(?<term>$term_regex)$/is))) {
+		$currOffset = $offset+length($`);
+		print_brat_output(\*TAGGEDCORPUS, \$termId, $+{term}, $currOffset, $currOffset+length($+{term}));
+	    }
+	}
+	$i++;
+    }
+    print STDERR "\n";
+
+    close TAGGEDCORPUS;
+#########################################################################################################
+    warn "\nEnd of term tagging\n";
+}
+
+sub print_brat_output() {
+    my ($descriptor, $termId, $matching_term, $start_offset, $end_offset) = @_;
+
+    print $descriptor "T$$termId\tterm $start_offset $end_offset\t$matching_term\n";
+    $$termId++;
+}
+
+
 sub printMatchingTerm_tab() {
     my ($ref_matching_term, $term, $sent_id, $ref_tab_results) = @_;
 
     my $tmp_line = "";
     my $tmp_key;
 
-    # warn "\nOK\n";
+    # warn "\nOK: $term\n";
+    # warn "ref_matching_term: " . join ("\t", @$ref_matching_term) . "\n";
 
     if (ref($ref_tab_results) eq "ARRAY") {
 	$tmp_line .= "$sent_id\t";
  	$tmp_line .= join ("\t", @$ref_matching_term);
 	push @$ref_tab_results, $tmp_line;
+	# warn "tmp_line: $tmp_line\n";
     } else {
 	if (ref($ref_tab_results) eq "HASH") {
 	    my @tab_tmp;
@@ -456,9 +654,24 @@ sub printMatchingTerm_tab() {
 
 	    push @tab_tmp, $sent_id;
 	    push @tab_tmp, @$ref_matching_term;
+	    push @tab_tmp, $term;
 
+	    # warn "term_key: $tmp_key\n";
 	    # if (!exists $ref_tab_results->{$tmp_key}) {
-	    $ref_tab_results->{$tmp_key} = \@tab_tmp;
+	    if (!exists($ref_tab_results->{$tmp_key})) {
+		# warn "!exists\n";
+		$ref_tab_results->{$tmp_key} = \@tab_tmp;
+	    } else {
+		# warn "exists\n";
+#		push @{$ref_tab_results->{$tmp_key}}, @tab_tmp;
+		if (defined $tab_tmp[3]) {
+		    $ref_tab_results->{$tmp_key}->[3] .= ";" . $tab_tmp[3];
+		} else {
+		    $ref_tab_results->{$tmp_key}->[3] .= ";";
+		}
+	    }
+	    # warn "tab_tmp: " . join ("\t", @{$ref_tab_results->{$tmp_key}}) . "\n";
+
 	    # } else {
 	    # 	foreach $refmatch (@{$ref_tab_results->{$tmp_key}}) {
 		    
@@ -530,6 +743,30 @@ is carried out only for the terms with a number of characters lesser
 or equal to C<$caseSensitive>.
 
 
+=head2 termtagging_brat()
+
+    termtagging_brat($corpus_filename, $term_list_filename, $output_filename, $lemmatised_corpus_filename, $caseSensitive);
+
+This is the main method of module. It loads the term list
+(C<$term_list_filename>) and tags the text corpus
+(C<$corpus_filename>). The output can be read by Brat (<http://brat.nlplab.org/>).
+
+It produces the list of matching terms and the sentence offset (and
+additional information given in the input file) where the terms can be
+found. The file C<$output_filename> contains this output.  To look up
+the lemmatised term (as a concatenation of lemmatised word), the
+lemmatised corpus C<$lemmatised_corpus_filename> has to be specified
+as fourth argument of the method.
+
+The parameter C<$caseSensitive> indicates if the term matching is case
+sensitive (value greater or equal to 0) or insensitive ((value
+strictly lesser than 0). If the value of C<$caseSensitive> is equal to
+0, the case sensitive match is carried out for any terms. If the value of
+C<$caseSensitive> is strictly greater than 0, the case sensitive match
+is carried out only for the terms with a number of characters lesser
+or equal to C<$caseSensitive>.
+
+
 =head2 load_TermList()
 
     load_TermList($term_list_filename,\@term_list);
@@ -612,6 +849,24 @@ C<$caseSensitive> is strictly greater than 0, the case sensitive match
 is carried out only for the terms with a number of characters lesser
 or equal to C<$caseSensitive>.
 
+=head2 term_tagging_offset_brat()
+
+    term_tagging_offset_brat(\@term_list, \@regex_term_list, \%idtrm_select, \%corpus, $output_filename, $caseSensitive);
+
+This method tags the corpus C<\%corpus> with the terms (issued from
+the term list C<\@term_list>, C<\@regex_term_list> is the term list
+with regular expression), and selected in a previous step
+(C<\%idtrm_select>). Resulting selected terms are recorded with their
+offset, and additional information in the file C<$output_filename> in the Brat input format (<http://brat.nlplab.org/>).
+
+The parameter C<$caseSensitive> indicates if the term matching is case
+sensitive (value greater or equal to 0) or insensitive ((value
+strictly lesser than 0). If the value of C<$caseSensitive> is equal to
+0, the case sensitive match is carried out for any terms. If the value of
+C<$caseSensitive> is strictly greater than 0, the case sensitive match
+is carried out only for the terms with a number of characters lesser
+or equal to C<$caseSensitive>.
+
 =head2 term_tagging_offset_tab()
 
     term_tagging_offset_tab(\@term_list, \@regex_term_list, \%idtrm_select, \%corpus, \@tab_results, $caseSensitive);
@@ -647,6 +902,16 @@ sentence id (C<$sentence_id>) and the matching term (named by its
 reference C<$ref_matching_term>). Both data are on a line and are
 separated by a tabulation character.
 
+=head2 print_brat_output
+
+    print_brat_output($descriptor, $termId, $matching_term, $start_offset, $end_offset);
+
+This method prints into the file descriptor C<$descriptor>, the term
+id (C<$termId>), the start and end offset of the term
+(C<$start_offset> and C<$end_offset>) and the matching term (named by
+its reference C<$matching_term>) in the Brat input. Both data are on a
+line and are separated by a tabulation character.
+
 =head2 printMatchingTerm_tab
 
     printMatchingTerm_tab($ref_matching_term, $sentence_id, $ref_tab_results);
@@ -660,13 +925,17 @@ data are stored in an array, hash key is the concatenation of the
 sentence id and the matching term.
 
 
+=head2 
+
 =head1 SEE ALSO
 
 Alvis web site: http://www.alvis.info
 
+Brat: http://brat.nlplab.org/
+
 =head1 AUTHORS
 
-Thierry Hamon <thierry.hamon@univ-paris13.fr>
+Thierry Hamon <thierry.hamon@limsi.fr>
 
 =head1 LICENSE
 
